import os
import typing as T
from .req_impl import Loader, _one_of, LibRequired

class YAMLLoader(Loader):
    """
    Builtin loader for YAML files
    """
    def __init__(self):
        self.yaml = _one_of(["yaml"])
        self.extensions = ['.yaml', '.yml']
        self.deps = ['yaml']
        self.optional_deps = {}
    
    def load(self, path: str, file: str) -> T.Dict[str, T.Any]:
        if not self.yaml:
            raise LibRequired("The yaml library is required to load .yaml files") from None
        try:
            with open(os.path.abspath(path) + '/' + file, 'r') as fp:
                return self.yaml.loads(fp.read(), Loader=self.yaml.loader.FullLoader)
        except FileNotFoundError:
            raise ValueError(f"No such file: {file}") from None

loader = YAMLLoader


import os
import typing as T
from .req_impl import Loader, _one_of

class JSONLoader(Loader):
    """
    Builtin loader for JSON files
    """
    def __init__(self):
        self.json = _one_of(["orjson", "ujson", "json"])
        self.extensions = ['.json']
        self.deps = []
        self.optional_deps = {
            "orjson": "Speedup",
            "ujson": "Alternative for orjson"
        }
    
    def load(self, path: str, file: str) -> T.Dict[str, T.Any]:
        try:
            with open(os.path.abspath(path) + '/' + file, 'rb') as fp:
                return self.json.loads(fp.read())
        except FileNotFoundError:
            raise ValueError(f"No such file: {file}") from None

loader = JSONLoader


from setuptools import setup

setup(
    name = "node_require",
    author = "weerdy15",
    url = "https://github.com/weerdy15/node-require",
    project_urls = {
        "Issue tracker": "https://github.com/weerdy15/node-require/issues"
    },
    version = "2.0.0",
    packages = [
        "node_require"
    ],
    license = "MIT",
    description = "Like Node.js's require(), but with more supported formats",
    long_description = open('README.md', 'r').read(),
    long_description_content_type = "text/markdown",
    install_requires = [],
    extras_require = {
        "toml": [ "toml" ],
        "yaml": [ "yaml" ],
        "bson": [ "bson" ],
        "all_langs": [ "toml", "yaml", "bson" ],
    },
    python_requires = ">=3.9.7",
    classifiers = [
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Utilities",
        "Typing :: Typed",
    ]
)


import os
import re
import typing as T
import importlib as ilib
from pip._internal.commands.show import search_packages_info as spi
# import warnings

_almost_any: str = "[a-zA-Z0-9_\\./\\$&;!-#@~`\\^%\\{\\}\\[\\]\\(\\)\\+ ]"
_l: str = "[a-zA-Z0-9_\\$&;!-#@~`\\^%\\{\\}\\[\\]\\(\\) ]"
QUERY: re.Pattern = re.compile(f"({_almost_any})+({_l})*\.({_almost_any})+")

T.Module = T.TypeVar("module", os.__class__, os.__class__)

loaders = {}

pkg = None
for entry in spi(['node_require']):
    pkg = entry
    break

class ExtNotSupported(Exception):
    """
    Exception that is raised when you trying to load unsupported file extension
    """
    pass

class LibRequired(Exception):
    """
    Exception that is raised when you trying to load extension that is supported, but dependencies for it are not resolved
    """

class Loader:
    """
    Abstract base class (ABC) for all loaders.
    """
    extensions: T.List[str]
    deps: T.List[str]
    optional_deps: T.Dict[str, str]
    
    def load(self, path: str, file: str) -> T.Any:
        """
        A function to be overridden by developer.
        """
        raise NotImplemented

def require(q: str) -> T.Any:
    """
    Like a Node's ``require()``
    
    Parameters
    ----------
    q: str
        The query, e.g., "../mymodule.py" or "./config.json"
    
    Returns
    -------
    Any
        The type depends on file you required. For example, if you required a Python file, it will return a Python module (if .py loader was not overridden)
    """
    # some checking
    if not QUERY.match(q):
        warnings.warn("It looks like you given an invalid path to file.")
    path, file = _parse(q)
    piece = _import(path, file)
    return piece

def _parse(q: str) -> T.Tuple[str, str]:
    newpart = False
    parts = []
    s = ""
    for x in q:
        if newpart:
            newpart = False
            parts.append(s)
            s = ""
        s += x
        if x == '/':
            newpart = True
    parts.append(s)
    file = parts.pop(len(parts) - 1)
    path = ''.join(parts)
    return (path, file)

def _import(path, file) -> T.Any:
    for x in list(loaders.keys()):
        if file.endswith(x):
            r = loaders[x].load(path, file)
            return r
    raise ExtNotSupported("The file you required ({file}) have unsupported extension")

def use(loader: Loader) -> None:
    """
    Register loader.
    
    Parameters
    ----------
    loader: Loader
       The loader
    """
    l = loader()
    for ext in l.extensions:
        loaders[ext] = l

def one_of(q: T.List[str]) -> T.Module:
    """
    Tries to import one of requested modules.
    """
    for module in q:
        try:
            return __import__(module)
        except ImportError:
            continue
    raise LibRequired("Please install one of the following libraries to use this loader: " + ", ".join(q))

def _one_of(q: T.List[str]) -> T.Module:
    """
    Silent version of ``one_of``. **Internal use only!**
    """
    try:
        return one_of(q)
    except LibRequired:
        return None

for loader_file in os.listdir(pkg.location + "/node_require"):
    if loader_file.endswith('_loader.py'):
        use(ilib.import_module(f".{loader_file[:-3]}", package="node_require").loader)


import os
import sys
from .req_impl import Loader

class ModuleLoader(Loader):
    """
    Builtin loader for Python modules
    """
    def __init__(self):
        self.extensions = ['.py']
        self.deps = []
        self.optional_deps = {}
    
    def load(self, path: str, file: str):
        r = None
        old = sys.path.copy()
        sys.path = [os.path.abspath(path)]
        try:
            r = __import__(file[:-3])
        except ModuleNotFoundError:
            sys.path = old
            raise ValueError(f"No such module: {file}") from None
        sys.path = old
        return r

loader = ModuleLoader


import os
import typing as T
from .req_impl import Loader, _one_of, LibRequired

class TomlLoader(Loader):
    """
    Builtin loader for Toml files
    """
    def __init__(self):
        self.toml = _one_of(["toml"])
        self.extensions = ['.toml']
        self.deps = ['toml']
        self.optional_deps = {}
    
    def load(self, path: str, file: str) -> T.Dict[str, T.Any]:
        if not self.toml:
            raise LibRequired("The toml library is required to load .toml files") from None
        try:
            with open(os.path.abspath(path) + '/' + file, 'r') as fp:
                return self.toml.loads(fp.read())
        except FileNotFoundError:
            raise ValueError(f"No such file: {file}") from None

loader = TomlLoader